import pytest
import os.path
from _fedmod.cli import ModtoolsCLI
from _fedmod.module_generator import ModuleGenerator

def _generate_modulemd(rpms, build_deps_iterations=0):
    # TODO: Actually test the CLI arg parsing via a subprocess
    cmd_input = list(['rpm2module'])
    cmd_input.extend(rpms)
    cli = ModtoolsCLI(cmd_input)
    mg = ModuleGenerator(cli.pkgs)
    if len(rpms) == 1:
        output_fname = rpms[0] + '.yaml'
    else:
        output_fname = 'modulemd-output.yaml'
    mg.run(output_fname, build_deps_iterations)
    return mg, output_fname


class TestSinglePackageInput(object):

    def setup(self):
        self.input_rpms = input_rpms = ('grep',)
        self.md, self.output_fname = _generate_modulemd(input_rpms)

    def teardown(self):
        os.remove(self.output_fname)

    def test_generated_modulemd_file(self):
        # File exists with expected name
        assert os.path.isfile(self.input_rpms[0] + '.yaml')

        # Check details of generated module metadata
        modmd = self.md.mmd

        # Expected description for 'grep'
        assert modmd.summary == "Pattern matching utilities"
        assert modmd.description == (
            "The GNU versions of commonly used grep utilities. Grep searches through\n"
            "textual input for lines which contain a match to a specified pattern and then\n"
            "prints the matching lines. GNU\'s grep utilities include grep, egrep and fgrep.\n\n"
            "GNU grep is needed by many scripts, so it shall be installed on every system."
        )

        # Expected licenses for 'grep'
        assert modmd.module_licenses == {'MIT'}
        assert modmd.content_licenses == {'GPLv3+'}

        # Only given modules are listed in the public API
        assert sorted(modmd.api.rpms) == sorted(self.input_rpms)

        # Expected components for grep
        assert set(modmd.components.rpms) == set(self.input_rpms)

        # Expected module dependencies for grep
        assert set(modmd.buildrequires) == set()
        assert set(modmd.requires) == {'platform',}



class TestMultiplePackageInput(object):

    def setup(self):
        self.input_rpms = input_rpms = ('grep', 'haproxy')
        self.md, self.output_fname = _generate_modulemd(input_rpms)

    def teardown(self):
        os.remove(self.output_fname)

    def test_generated_modulemd_file(self):
        # File exists with expected name
        assert os.path.isfile('modulemd-output.yaml')

        # Check details of generated module metadata
        modmd = self.md.mmd

        # Can't generate descriptive metadata when given multiple RPMs
        assert modmd.summary == ""
        assert modmd.description == ""

        # Expected licenses for grep + haproxy
        assert modmd.module_licenses == {'MIT'}
        assert len(modmd.content_licenses) == 0 # This doesn't seem right...

        # Only given modules are listed in the public API
        assert sorted(modmd.api.rpms) == sorted(self.input_rpms)

        # Expected components for grep + haproxy
        expected_components = set(self.input_rpms)
        assert set(modmd.components.rpms) == expected_components

        # Expected module dependencies for grep + haproxy
        assert set(modmd.buildrequires) == {'platform',}
        assert set(modmd.requires) == {'platform',}


class TestRecursiveBuildDeps(object):

    def setup(self):
        self.input_rpms = input_rpms = ('mariadb',)
        self.md, self.output_fname = _generate_modulemd(input_rpms, 100)

    def teardown(self):
        os.remove(self.output_fname)

    def test_generated_modulemd_file(self):
        # File exists with expected name
        assert os.path.isfile('mariadb.yaml')

        # Check details of generated module metadata
        modmd = self.md.mmd

        # Descriptive metadata
        assert modmd.summary == "A community developed branch of MySQL"
        assert modmd.description == (
            "MariaDB is a community developed branch of MySQL.\n"
            "MariaDB is a multi-user, multi-threaded SQL database server.\n"
            "It is a client/server implementation consisting of a server daemon (mysqld)\n"
            "and many different client programs and libraries. The base package\n"
            "contains the standard MariaDB/MySQL client programs and generic MySQL files."
        )

        # Expected licenses
        assert modmd.module_licenses == {'MIT'}
        assert modmd.content_licenses == {'GPLv2 with exceptions and LGPLv2 and BSD'}

        # Only given modules are listed in the public API
        assert sorted(modmd.api.rpms) == sorted(self.input_rpms)

        # MariaDB's complicated test suite poses some real challenges for
        # build dependency resolution, even when the generator is kind of
        # cheating and relying on the actual Fedora MariaDB module at runtime

        # Expected components
        expected_components = set(self.input_rpms)
        expected_components |= {
            'systemtap',
            'multilib-rpm-config',
            'libselinux',
            'libuv',
            'jsoncpp',
            'python-nose',
            'numpy',
            'openblas',
            'Judy',
            'setools',
            'rhash',
            'pyparsing',
            'boost',
            'libsemanage',
            'jemalloc',
        }
        assert set(modmd.components.rpms) == expected_components

        # Expected module dependencies
        expected_build_deps = {
            'platform',
            'mariadb',
            'python2',
            'networking-base',
            'host',
            'python3',
            'perl',
            'installer',
            'mariadb',
        }
        assert set(modmd.buildrequires) == expected_build_deps
        assert set(modmd.requires) == {'platform', 'perl', 'mariadb'}


class TestNonRecursiveBuildDeps(object):

    def setup(self):
        self.input_rpms = input_rpms = ('graphite-web',)
        self.md, self.output_fname = _generate_modulemd(input_rpms)

    def teardown(self):
        os.remove(self.output_fname)

    def test_generated_modulemd_file(self):
        # File exists with expected name
        assert os.path.isfile('graphite-web.yaml')

        # Check details of generated module metadata
        modmd = self.md.mmd

        # Descriptive metadata
        assert modmd.summary == "A Django web application for enterprise scalable realtime graphing"
        assert modmd.description == (
            "Graphite consists of a storage backend and a web-based visualization frontend.\n"
            "Client applications send streams of numeric time-series data to the Graphite\n"
            "backend (called carbon), where it gets stored in fixed-size database files\n"
            "similar in design to RRD. The web frontend provides user interfaces\n"
            "for visualizing this data in graphs as well as a simple URL-based API for\n"
            "direct graph generation.\n\n"
            "Graphite's design is focused on providing simple interfaces (both to users and\n"
            "applications), real-time visualization, high-availability, and enterprise\n"
            "scalability."
        )

        # Expected licenses
        assert modmd.module_licenses == {'MIT'}
        assert modmd.content_licenses == {'ASL 2.0'}

        # Only given modules are listed in the public API
        assert sorted(modmd.api.rpms) == sorted(self.input_rpms)

        # Expected components
        expected_components = set(self.input_rpms)
        expected_components |= {
            'python-twisted',
            'python-simplejson',
            'python-service-identity',
            'python-crypto',
            'python-xpyb',
            'python-memcached',
            'python-zope-interface',
            'python-whitenoise',
            'python-whisper',
            'pyparsing',
            'dejavu-fonts',
            'python-zope-event',
            'python-django',
            'python-django-tagging',
            'python-attrs',
            'python-carbon',
            'python-fadvise',
            'pyserial',
        }
        assert set(modmd.components.rpms) == expected_components

        # Expected module dependencies
        # graphite-web is currently generating runtime module dependencies
        # than expected, so it makes for an interesting test case to look for
        # cases where fedmod is picking up module implementation details

        expected_build_requires = {
            'platform',
            'fonts',
            'python2',
            'python2-ecosystem'
        }
        _unexpected_build_requires = {
            'pki',
            'samba',
            'python3',
            'python3-ecosystem',
            'installer',
            'X11-base',
            'freeipa',
            'cloud-init',
            'perl',
            'fonts',
            'freeipa',
            'java',
            'host',
            'networking-base',
        }
        assert set(modmd.buildrequires) == expected_build_requires | _unexpected_build_requires
        expected_requires = {
            'platform',
            'httpd',
            'fonts',
            'python2',
        }
        _unexpected_requires = {
            'pki',
            'samba',
            'python3-ecosystem',
            'installer',
            'X11-base',
            'freeipa',
        }
        assert set(modmd.requires) == expected_requires | _unexpected_requires
